@@ -5,8 +5,12 @@ from kks.util import get_valid_session, load_links
 
 
 @click.command()
-def status():
-    """Parse and display task status"""
+@click.argument('filters', nargs=-1)
+def status(filters):
+    """Parse and display task status
+
+    If any FILTERS are specified, show status only for tasks with matching prefixes/names
+    """
 
     session = get_valid_session()
     if session is None:
@@ -18,10 +22,16 @@ def status():
         return
 
     problems = ejudge_summary(links, session)
+    if filters:
+        problems = [p for p in problems if any(p.short_name.startswith(f) for f in filters)]
+        if not problems:
+            click.secho('Nothing found')
+            return
 
-    row_format = "{:8} {:30} {:20} {:>5}"
 
-    click.secho(row_format.format("Alias", "Name", "Status", "Score"), fg='green', bold=True)
+    row_format = "{:8} {:35} {:20} {:>5}"
+
+    click.secho(row_format.format("Alias", "Name", "Status", "Score"), bold=True)
 
     for problem in problems:
         string = row_format\
